# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['leetscrape']

package_data = \
{'': ['*']}

install_requires = \
['black>=22.12.0,<23.0.0',
 'docstring-parser>=0.15,<0.16',
 'markdownify>=0.11.6,<0.12.0',
 'marko>=1.2.2,<2.0.0',
 'pandas>=1.5.2,<2.0.0',
 'pydantic>=1.10.4,<2.0.0',
 'requests>=2.28.2,<3.0.0',
 'tqdm>=4.64.1,<5.0.0']

entry_points = \
{'console_scripts': ['leetscrape = leetscrape.scripts:leetscrape',
                     'leetupload_solution = '
                     'leetscrape.scripts:leetupload_solution']}

setup_kwargs = {
    'name': 'leetscrape',
    'version': '1.0.1',
    'description': 'Introducing LeetScrape - a powerful and efficient Python package designed to scrape problem statements and their topic and company tags, difficulty, test cases, hints, and code stubs from LeetCode.com. Easily download and save LeetCode problems to your local machine, making it convenient for offline practice and studying. It is perfect for anyone preparing for coding interviews. With the LeetScrape, you can boost your coding skills and improve your chances of landing your dream job.',
    'long_description': '# LeetScrape\n\n[![deploy-docs](https://github.com/nikhil-ravi/LeetScrape/actions/workflows/pytest-and-docs.yml/badge.svg)](https://leetscrape.chowkabhara.com) [![PYPI](https://img.shields.io/pypi/v/leetscrape)](https://pypi.org/project/leetscrape/) [![codecov](https://codecov.io/gh/nikhil-ravi/LeetScrape/branch/main/graph/badge.svg?token=GWOVLPYSUA)](https://codecov.io/gh/nikhil-ravi/LeetScrape)![PyPI - Downloads](https://img.shields.io/pypi/dm/leetscrape)\n\nIntroducing the LeetScrape - a powerful and efficient Python package designed to scrape problem statements and basic test cases from LeetCode.com. With this package, you can easily download and save LeetCode problems to your local machine, making it convenient for offline practice and studying. It is perfect for software engineers and students preparing for coding interviews. The package is lightweight, easy to use and can be integrated with other tools and IDEs. With the LeetScrape, you can boost your coding skills and improve your chances of landing your dream job.\n\nUse this package to get the list of Leetcode questions, their topic and company tags, difficulty, question body (including test cases, constraints, hints), and code stubs in any of the available programming languages.\n\nDetailed documentation available [here](https://leetscrape.chowkabhara.com/).\n\nThere is also a related Next.js web app to serve the scraped questions and your answers at [leetcode-nextjs](https://github.com/nikhil-ravi/leetscrape-ts). See the [demo](https://scuffedcode.chowkabhara.com/).\n\n## Get Started\n\n### Installation\n\nStart by installing the package from pip or conda:\n\n```bash\n# Using pip\npip install leetscrape\n\n# Using conda:\nconda install leetscrape\n\n# Using poetry:\npoetry add leetscrape\n```\n\n### Usage\n\n#### Command Line\n\n* <code>leetscrape <i><b>list</b></i> [--out OUT]</code> - List all questions without generating code stub.\n\n    ```\n    options:\n    -h, --help         show a help message and exit\n    --out OUT, -o OUT  Specify the output file name to store the list of questions.\n    ```\n* <code>leetscrape <i><b>question</b></i> [--out OUT] qid [qid ...]</code> - Generate a code stub for the given question(s).\n\n    ```\n    positional arguments:\n    qid                Enter Leetcode question ID(s)\n\n    options:\n    -h, --help         show this help message and exit\n    --out OUT, -o OUT  Enter the path to the output directory\n    ```\n* <code>leetscrape <i><b>solution</b></i> [-h] [--out OUT] input</code> - Generate mdx files from solutions.\n\n    ```\n    positional arguments:\n    input              Enter the path to the solution directory with solution files or to a single\n                        solution file\n\n    options:\n    -h, --help         show this help message and exit\n    --out OUT, -o OUT  Enter the path to the output directory to save solutions mdx files\n    ```\n* <code>leetscrape <i><b>ts</b></i> [--out OUT]</code> - Create the leetscrape-ts Next.js project to host the solutions.\n\n    ```\n    options:\n    -h, --help         show this help message and exit\n    --out OUT, -o OUT  Enter the path to the output directory to save the project\n    ```\n\n#### Python API\n\n##### Get the list of problems and their information\n\n```python\nfrom leetscrape import GetQuestionsList\n\nls = GetQuestionsList()\nls.scrape() # Scrape the list of questions\nls.questions.head() # Get the list of questions\n```\n\n|    |   QID | title                                          | titleSlug                                      | difficulty   |   acceptanceRate | paidOnly   | topicTags                              | categorySlug   |\n|---:|------:|:-----------------------------------------------|:-----------------------------------------------|:-------------|-----------------:|:-----------|:---------------------------------------|:---------------|\n|  0 |     1 | Two Sum                                        | two-sum                                        | Easy         |          51.4225 | False      | array,hash-table                       | algorithms     |\n|  1 |     2 | Add Two Numbers                                | add-two-numbers                                | Medium       |          41.9051 | False      | linked-list,math,recursion             | algorithms     |\n|  2 |     3 | Longest Substring Without Repeating Characters | longest-substring-without-repeating-characters | Medium       |          34.3169 | False      | hash-table,string,sliding-window       | algorithms     |\n|  3 |     4 | Median of Two Sorted Arrays                    | median-of-two-sorted-arrays                    | Hard         |          38.8566 | False      | array,binary-search,divide-and-conquer | algorithms     |\n|  4 |     5 | Longest Palindromic Substring                  | longest-palindromic-substring                  | Medium       |          33.4383 | False      | string,dynamic-programming             | algorithms     |\n\nYou can export the associated tables to a directory using the `to_csv` method:\n\n```python\nls.to_csv(directory="<dir>")\n```\nThis generates 6 `.csv` files in the current directory:\n- `questions.csv` - List of questions with their title, difficulty, acceptance rate, paid status, topic tags, and category.\n- `companies.csv` - List of companies with their name, slug, and the questions count.\n- `topicsTags.csv` - List of topic tags with their name and slug.\n- `categories.csv` - List of categories with their name and slug.\n- `questionCategory.csv` - An edgelist of questions and their categories.\n- `questionTopics.csv` - An edgelist of questions and their topic tags.\n\n##### Get Question statement and other information\n\nQuery individual question\'s information such as the body, test cases, constraints, hints, code stubs, and company tags using the `GetQuestion` class:\n\n```python\nfrom leetscrape import GetQuestion\n\n# Get the question body\nquestion = GetQuestion(titleSlug="two-sum").scrape()\n```\n\nThis returns a `Question` object with the following attributes:\n\n```python\nquestion.QID # Question ID\nquestion.title # Question title\nquestion.titleSlug # Question title slug\nquestion.difficulty # Question difficulty\nquestion.Hints # Question hints\nquestion.Companies # Question companies\nquestion.topics # Question topic tags\nquestion.SimilarQuestions # Similar questions ids\nquestion.Code # Code stubs\nquestion.Body # Question body / problem statement\nquestion.isPaidOnly # Whether the question is only available to premium users of Leetcode\n```\n\n##### Generate code stubs for a question\n\n```python\nfrom leetscrape import GenerateCodeStub\n\n# Get the question body\nfcs = GenerateCodeStub(titleSlug="two-sum")\nfcs.generate(directory="<dir>")\n```\nThis generates the following files in the given directory:\n- `q_0001_twoSum.py` - Python file with the code stub for the given question with a function named `twoSum`.\n- `test_q_0001_twoSum.py` - Python file with the test cases for the given question.\n\nSee [examples](./example/solutions/) for examples of the generated code stubs.\n\n##### Generate mdx files from solutions\n\nOnce you have solved a question, you can generate an mdx file with the solution and the question statement using the `ExtractSolutions` class:\n\n```python\nfrom leetscrape import ExtractSolutions\n\n# Get the question body\nsolutions = ExtractSolutions(filename="<path-to-solution-file>").extract()\n```\nThis outputs a list of `Solution` objects with the following attributes:\n\n```python\nsolution.id # Solution ID\nsolution.code # Solution code\nsolution.docs # Docstrings associated with the solution\nsolution.problem_statement # Question body / problem statement\n```\n\nAlternatively, you can use the `to_mdx` method to generate the mdx file:\n\n```python\nfrom leetscrape import ExtractSolutions\n\n# Get the question body\nExtractSolutions(filename="<path-to-solution-file>").to_mdx(output_filename="<path-to-output-file>")\n```\n\n### Serving the solutions with [leetscrape-ts](https://github.com/nikhil-ravi/leetscrape-ts)\n\nYou can use the [leetscrape-ts](https://github.com/nikhil-ravi/leetscrape-ts) Next.js template to serve the solutions. See the [demo](https://scuffedcode.chowkabhara.com/). Visit the repo for more details. You can generate the project using the `leetscrape ts` command:\n\n```bash\nleetscrape ts --out <path-to-output-directory>\n```\nThis will bootstrap the project in the given directory. Follow the instructions in the [README](https://github.com/nikhil-ravi/leetscrape-ts/blob/main/README.md) and create/modify the `.env.local` file. Then, run the following command to generate the mdx files:\n\n```bash\nleetscrape solution --out <path-to-output-directory>/src/content/solutions <path-to-your-python-solution-directory>\n```\n\nYou can then run the project using the following command:\n\n```bash\ncd <path-to-output-directory>\nnpm run dev\n# or\nyarn dev\n# or\npnpm dev\n# or\nbun dev\n```\n\n## Changelog\n\nView the changelog [here](https://leetscrape.chowkabhara.com/CHANGELOG/).',
    'author': 'Nikhil Ravi',
    'author_email': 'nr337@cornell.edu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/nikhil-ravi/LeetScrape',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
